﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// An operator which creates a new random permutation of integer values.
  /// </summary>
  [Item("RandomParameterSetCreator", "An operator which creates a new set of parameters TODO.")]
  [StorableClass]
  public sealed class RandomParameterSetCreator : SingleSuccessorOperator, IParameterSetCreator {
    public override bool CanChangeName {
      get { return false; }
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<IParameterSet> ParameterSetParameter {
      get { return (ILookupParameter<IParameterSet>)Parameters["ParameterSet"]; }
    }

    [Storable]
    private ParameterConfigurationList parametersToOptimize;
    public ParameterConfigurationList ParametersToOptimize {
      get { return parametersToOptimize; }
      set { parametersToOptimize = value; }
    }

    public RandomParameterSetCreator()
      : base() {
        Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used to initialize the new random permutation."));
        Parameters.Add(new LookupParameter<IParameterSet>("ParameterSet", "The new random parameter set."));
    }

    public override IOperation Apply() {
      ParameterSetParameter.ActualValue = new ParameterSet(ParametersToOptimize, RandomParameter.ActualValue);
      return base.Apply();
    }

    #region Cloning
    public override IDeepCloneable Clone(Cloner cloner) {
      RandomParameterSetCreator clone = (RandomParameterSetCreator) base.Clone(cloner);
      clone.parametersToOptimize = (ParameterConfigurationList)this.parametersToOptimize.Clone();
      return clone;
    }
    #endregion
  }
}
